/*
 * =================================================================================
 * Copyright (C) 2013 Martin Albedinsky [Wolf-ITechnologies]
 * =================================================================================
 * Licensed under the Apache License, Version 2.0 or later (further "License" only);
 * ---------------------------------------------------------------------------------
 * You may use this file only in compliance with the License. More details and copy
 * of this License you may obtain at
 * 
 * 		http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You can redistribute, modify or publish any part of the code written in this
 * file but as it is described in the License, the software distributed under the 
 * License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES or CONDITIONS OF
 * ANY KIND.
 * 
 * See the License for the specific language governing permissions and limitations
 * under the License.
 * =================================================================================
 */
package com.wit.and.database.test;

import android.test.suitebuilder.annotation.SmallTest;
import android.util.SparseArray;

import com.wit.and.database.BaseDatabaseProvider;
import com.wit.and.database.annotations.DatabaseAuthority;
import com.wit.and.database.annotations.DatabaseName;
import com.wit.and.database.annotations.DatabaseVersion;

/**
 * <p>
 * Tests logic of {@link } in {@link android.test.AndroidTestCase} environment.
 * </p>
 *
 * @author Martin Albedinsky
 */
public class BaseDatabaseProviderTest extends android.test.AndroidTestCase {

	/**
	 * Test Constants =============================
	 */

	/**
	 * Debug/Test TAG.
	 */
	private static final String TAG = BaseDatabaseProviderTest.class.getSimpleName();

	/**
	 * Test Members ===============================
	 */

	private final DatabaseProvider PROVIDER = new DatabaseProvider();

	/**
	 * Methods ====================================
	 */

	/**
	 * Set-up methods -----------------------------
	 */

	/**
	 * Set ups before each test.
	 *
	 * @throws Exception
	 */
	@Override
	protected void setUp() throws Exception {
		super.setUp();
	}

	/**
	 * Test methods -------------------------------
	 */

	@SmallTest
	public void testSetMaxCachedTablesCount() {
		DatabaseProvider.setMaxCachedEntitiesCount(25);
		assertEquals(25, DatabaseProvider.getMaxCachedEntitiesCount());

		DatabaseProvider.setMaxCachedEntitiesCount(100);
		assertEquals(DatabaseProvider.MAX_CACHED_ENTITIES, DatabaseProvider.getMaxCachedEntitiesCount());

		final int current = DatabaseProvider.getMaxCachedEntitiesCount();
		DatabaseProvider.setMaxCachedEntitiesCount(-5);
		assertEquals(current, DatabaseProvider.getMaxCachedEntitiesCount());
	}

	@SmallTest
	public void testGetDatabaseAuthority() {
		assertEquals(DatabaseProvider.DATABASE_AUTHORITY, PROVIDER.getDatabaseAuthority());
	}

	@SmallTest
	public void testGetDatabaseVersion() {
		assertEquals(DatabaseProvider.DATABASE_VERSION, PROVIDER.getDatabaseVersion());
	}

	@SmallTest
	public void testGetDatabaseName() {
		assertEquals(DatabaseProvider.DATABASE_NAME, PROVIDER.getDatabaseName());
	}

	/**
	 * Implementation -----------------------------
	 */

	/**
	 * Simple implementation of database provider.
	 */
	@DatabaseName(DatabaseProvider.DATABASE_NAME)
	@DatabaseVersion(DatabaseProvider.DATABASE_VERSION)
	@DatabaseAuthority(DatabaseProvider.DATABASE_AUTHORITY)
	//@DatabaseEntities({TableCustomers.class, TableGoods.class})
	public static final class DatabaseProvider extends BaseDatabaseProvider {

		public static final String DATABASE_AUTHORITY = "test.db";

		public static final String DATABASE_NAME = "com.wit.and.database.test";

		public static final int DATABASE_VERSION = 1;

		@Override
		public SparseArray<Class<? extends BaseDatabaseEntity>> getAssignedEntities() {
			return super.getAssignedEntities();
		}
	}
}