/*
 * =================================================================================
 * Copyright (C) 2013 Martin Albedinsky [Wolf-ITechnologies]
 * =================================================================================
 * Licensed under the Apache License, Version 2.0 or later (further "License" only);
 * ---------------------------------------------------------------------------------
 * You may use this file only in compliance with the License. More details and copy
 * of this License you may obtain at
 * 
 * 		http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You can redistribute, modify or publish any part of the code written in this
 * file but as it is described in the License, the software distributed under the 
 * License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES or CONDITIONS OF
 * ANY KIND.
 * 
 * See the License for the specific language governing permissions and limitations
 * under the License.
 * =================================================================================
 */
package com.wit.and.database.test.entity;

import android.content.ContentValues;
import android.test.suitebuilder.annotation.SmallTest;

import com.wit.and.database.annotations.entity.EntityName;
import com.wit.and.database.annotations.manage.ContentValue;
import com.wit.and.database.entity.DatabaseEntityModel;

/**
 * <p>
 * Tests logic of {@link com.wit.and.database.entity.DatabaseEntityModel} in the {@link android.test.AndroidTestCase} environment.
 * </p>
 *
 * @author Martin Albedinsky
 */
public class DatabaseEntityModelTest extends android.test.AndroidTestCase {

	/**
	 * Test Constants =============================
	 */

	/**
	 * Debug/Test TAG.
	 */
	private static final String TAG = DatabaseEntityModelTest.class.getSimpleName();

	/**
	 * Test Members ===============================
	 */

	/**
	 * Methods ====================================
	 */

	/**
	 * Set-up methods -----------------------------
	 */

	/**
	 * Test methods -------------------------------
	 */

	@SmallTest
	public void testCreateContentValuesSimple() {
		ContentValues contentValues = new SimpleModelTest("Some string", 0x02).createContentValues();
		assertNotNull(contentValues);
		assertTrue(contentValues.containsKey("column_string"));
		assertTrue(contentValues.containsKey("column_integer"));
		assertEquals("Some string", contentValues.getAsString("column_string"));
		assertEquals(Integer.valueOf(0x02), contentValues.getAsInteger("column_integer"));
	}

	@SmallTest
	public void testCreateContentValuesComplex() {
		ContentValues contentValues = new DatabaseEntityModel().toContentValues(new ComplexModelTest(ComplexModelTest.Type.SECOND));
		assertNotNull(contentValues);
		assertTrue(contentValues.containsKey("test_column_double"));
		assertTrue(contentValues.containsKey("test_column_type"));
		assertEquals(12.45d, contentValues.getAsDouble("test_column_double"));
		assertEquals(Integer.valueOf(ComplexModelTest.Type.SECOND.ordinal()), contentValues.getAsInteger("test_column_type"));
	}

	/**
	 * Test implementations -----------------------
	 */

	/**
	 * Simple implementation of database entity model.
	 */
	private static final class SimpleModelTest extends DatabaseEntityModel {

		@ContentValue("column_string")
		private String mString;

		@ContentValue("column_integer")
		private int mInteger;

		public SimpleModelTest(String string, int integer) {
			this.mString = string;
			this.mInteger = integer;
		}
	}

	/**
	 * Quite complex implementation of database entity model.
	 */
	@EntityName("test")
	private static final class ComplexModelTest {

		@ContentValue
		private Double column_double = 12.45d;

		@ContentValue
		private Type column_type;

		enum Type {
			FIRST, SECOND
		}

		public ComplexModelTest(Type type) {
			this.column_type = type;
		}
	}
}