/*
 * =================================================================================
 * Copyright (C) 2013 Martin Albedinsky [Wolf-ITechnologies]
 * =================================================================================
 * Licensed under the Apache License, Version 2.0 or later (further "License" only);
 * ---------------------------------------------------------------------------------
 * You may use this file only in compliance with the License. More details and copy
 * of this License you may obtain at
 * 
 * 		http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You can redistribute, modify or publish any part of the code written in this
 * file but as it is described in the License, the software distributed under the 
 * License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES or CONDITIONS OF
 * ANY KIND.
 * 
 * See the License for the specific language governing permissions and limitations
 * under the License.
 * =================================================================================
 */
package com.wit.and.database.test.manage;

import android.test.suitebuilder.annotation.SmallTest;

import com.wit.and.database.annotations.entity.EntityName;
import com.wit.and.database.annotations.manage.Columns;
import com.wit.and.database.entity.DatabaseEntity;
import com.wit.and.database.manage.EntityProjection;

/**
 * <p>
 * Tests logic of {@link } in the {@link android.test.AndroidTestCase} environment.
 * </p>
 *
 * @author Martin Albedinsky
 */
public class EntityProjectionTest extends android.test.AndroidTestCase {

	/**
	 * Test Constants =============================
	 */

	/**
	 * Debug/Test TAG.
	 */
	private static final String TAG = EntityProjectionTest.class.getSimpleName();

	/**
	 * Test Members ===============================
	 */

	/**
	 * Methods ====================================
	 */

	/**
	 * Set-up methods -----------------------------
	 */

	/**
	 * Test methods -------------------------------
	 */

	@SmallTest
	public void testCreateProjectionRow() {
		assertEquals("test_entity.column_1", EntityProjection.createProjectionRow("test_entity", "column_1"));
	}

	@SmallTest
	public void testGetTableName() {
		EntityProjection projection = new EntityProjection("test_entity");
		assertEquals("test_entity", projection.getTableName());
	}

	@SmallTest
	public void testGetTableNameComplex() {
		ComplexProjectionTest projection = new ComplexProjectionTest();
		assertEquals("complex_test_entity", projection.getTableName());
	}

	@SmallTest
	public void testGetTableNameComplexEnclosed() {
		TableTest.ComplexEnclosedProjectionTest projection = new TableTest.ComplexEnclosedProjectionTest();
		assertEquals("test_entity_with_enclosed_projection", projection.getTableName());
	}

	@SmallTest
	public void testColumn() {
		EntityProjection projection = new EntityProjection("test_entity");
		projection.column("column_1").column("column_2");
		assertEquals(2, projection.getColumns().size());
		assertEquals("column_1", projection.getColumns().get(0));
		assertEquals("column_2", projection.getColumns().get(1));
	}

	@SmallTest
	public void testColumns() {
		EntityProjection projection = new EntityProjection("test_entity");
		projection.columns("column_1", "column_2");
		assertEquals(2, projection.getColumns().size());
		assertEquals("column_1", projection.getColumns().get(0));
		assertEquals("column_2", projection.getColumns().get(1));
	}

	@SmallTest
	public void testGetColumnsComplex() {
		ComplexProjectionTest projection = new ComplexProjectionTest();
		assertEquals(3, projection.getColumns().size());
		assertEquals("complex_column_1", projection.getColumns().get(0));
		assertEquals("complex_column_2", projection.getColumns().get(1));
		assertEquals("complex_column_3", projection.getColumns().get(2));
	}

	@SmallTest
	public void testCreate() {
		EntityProjection projection = new EntityProjection("test_entity");
		assertEquals(0, projection.create().length);

		projection.columns("column_1", "column_2");
		String[] proj = projection.create();
		assertEquals(2, proj.length);
		assertEquals("test_entity.column_1", proj[0]);
		assertEquals("test_entity.column_2", proj[1]);
	}

	/**
	 * Test implementations -----------------------
	 */

	@EntityName("complex_test_entity")
	@Columns({"complex_column_1", "complex_column_2", "complex_column_3"})
	private static final class ComplexProjectionTest extends EntityProjection {}

	@EntityName("test_entity_with_enclosed_projection")
	private static final class TableTest extends DatabaseEntity {
		private static final class ComplexEnclosedProjectionTest extends EntityProjection {};
	}
}