/*
 * =================================================================================
 * Copyright (C) 2013 Martin Albedinsky [Wolf-ITechnologies]
 * =================================================================================
 * Licensed under the Apache License, Version 2.0 or later (further "License" only);
 * ---------------------------------------------------------------------------------
 * You may use this file only in compliance with the License. More details and copy
 * of this License you may obtain at
 * 
 * 		http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You can redistribute, modify or publish any part of the code written in this
 * file but as it is described in the License, the software distributed under the 
 * License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES or CONDITIONS OF
 * ANY KIND.
 * 
 * See the License for the specific language governing permissions and limitations
 * under the License.
 * =================================================================================
 */
package com.wit.and.database;

import android.database.Cursor;
import android.database.CursorWrapper;

/**
 * <h4>Class Overview</h4>
 * <p>
 * Description.
 * </p>
 *
 * @author Martin Albedinsky
 */
public abstract class BaseCursorWrapper extends CursorWrapper {

    /**
     * Constants =============================
     */

    /**
     * Log TAG.
     */
    private static final String TAG = BaseCursorWrapper.class.getSimpleName();

    /**
     * Indicates if debug private output trough log-cat is enabled.
     */
    // private static final boolean DEBUG = true;

    /**
     * Indicates if logging for user output trough log-cat is enabled.
     */
    // private static final boolean USER_LOG = true;

    /**
     * Enums =================================
     */

    /**
     * Static members ========================
     */

    /**
     * Members ===============================
     */

	/**
	 *
	 */
	private String mColumnIDName;

	/**
	 *
	 */
	private long mID = -1;

    /**
     * Listeners -----------------------------
     */

    /**
     * Arrays --------------------------------
     */

    /**
     * Booleans ------------------------------
     */

    /**
     * Constructors ==========================
     */

	/**
	 * <p>
	 * Same as {@link #BaseCursorWrapper(android.database.Cursor, String)}
	 * without primary key column name.
	 * </p>
	 *
	 * @param cursor Cursor to wrap.
	 */
    public BaseCursorWrapper(Cursor cursor) {
	    this(cursor, null);
    }

    /**
     * <p>
     * </p>
     *
     * @param cursor Cursor to wrap.
     * @param columnIDName Name of column holding the primary key.
     */
    public BaseCursorWrapper(Cursor cursor, String columnIDName) {
        super(cursor);

        if (cursor == null) {
            throw new IllegalArgumentException("Can't create cursor wrapper without valid cursor.");
        }
	    this.mColumnIDName = columnIDName;
        moveToFirst();
    }

    /**
     * Methods ===============================
     */

    /**
     * Public --------------------------------
     */

    /**
     */
    @Override
    public boolean move(int offset) {
        return super.move(offset) && performBindData();
    }

    /**
     */
    @Override
    public boolean moveToFirst() {
        return super.moveToFirst() && performBindData();
    }

    /**
     */
    @Override
    public boolean moveToLast() {
        return super.moveToLast() && performBindData();
    }

    /**
     */
    @Override
    public boolean moveToNext() {
        return super.moveToNext() && performBindData();
    }

    /**
     */
    @Override
    public boolean moveToPosition(int position) {
        return super.moveToPosition(position) && performBindData();
    }

    /**
     */
    @Override
    public boolean moveToPrevious() {
        return super.moveToPrevious() && performBindData();
    }

    /**
     * Getters + Setters ---------------------
     */

	/**
	 * <p>
	 * </p>
	 *
	 * @return
	 */
	public long getID() {
		return mID;
	}

    /**
     * Protected -----------------------------
     */

    /**
     * <p>
     * </p>
     *
     * @param columnName
     * @return
     */
    protected Short getShort(String columnName) {
        return super.getShort(getColumnIndex(columnName));
    }

    /**
     * <p>
     * </p>
     *
     * @param columnName
     * @return
     */
    protected String getString(String columnName) {
        return super.getString(getColumnIndex(columnName));
    }

    /**
     * <p>
     * </p>
     *
     * @param columnName
     * @return
     */
    protected int getInt(String columnName) {
        return super.getInt(getColumnIndex(columnName));
    }

    /**
     * <p>
     * </p>
     *
     * @param columnName
     * @return
     */
    protected float getFloat(String columnName) {
        return super.getFloat(getColumnIndex(columnName));
    }

    /**
     * <p>
     * </p>
     *
     * @param columnName
     * @return
     */
    protected long getLong(String columnName) {
        return super.getLong(getColumnIndex(columnName));
    }

    /**
     * <p>
     * </p>
     *
     * @param columnName
     * @return
     */
    protected double getDouble(String columnName) {
        return super.getDouble(getColumnIndex(columnName));
    }

    /**
     * Private -------------------------------
     */

    /**
     * Performs data binding.
     *
     * @return
     */
    private boolean performBindData() {
	    // Obtain primary key value.
	    this.mID = (mColumnIDName != null) ? getLong(mColumnIDName) : -1;
	    // Dispatch to bind.
        return onBindData();
    }

    /**
     * Abstract methods ----------------------
     */

    /**
     * <p>
     * Invoked to obtain data for the current cursor position.
     * </p>
     *
     * @return <code>True</code> if binding was successful and data for the current
     *         position are prepared, <code>false</code> otherwise.
     */
    protected abstract boolean onBindData();

    /**
     * Inner classes =========================
     */

    /**
     * Interface =============================
     */
}
