/*
 * =================================================================================
 * Copyright (C) 2013 Martin Albedinsky [Wolf-ITechnologies]
 * =================================================================================
 * Licensed under the Apache License, Version 2.0 or later (further "License" only);
 * ---------------------------------------------------------------------------------
 * You may use this file only in compliance with the License. More details and copy
 * of this License you may obtain at
 * 
 * 		http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You can redistribute, modify or publish any part of the code written in this
 * file but as it is described in the License, the software distributed under the 
 * License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES or CONDITIONS OF
 * ANY KIND.
 * 
 * See the License for the specific language governing permissions and limitations
 * under the License.
 * =================================================================================
 */
package com.wit.and.database.entity;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * <h4>Class Overview</h4>
 * <p>
 * </p>
 *
 * @author Martin Albedinsky
 */
public class DatabaseEntityColumn {

    /**
     * Constants =============================
     */

    /**
     * Log TAG.
     */
    private static final String TAG = DatabaseEntityColumn.class.getSimpleName();

    /**
     * Indicates if debug private output trough log-cat is enabled.
     */
    // private static final boolean DEBUG = true;

    /**
     * Indicates if logging for user output trough log-cat is enabled.
     */
    // private static final boolean USER_LOG = true;

    /**
     * Enums =================================
     */

    /**
     * Static members ========================
     */

    /**
     * Members ===============================
     */

	/**
	 *
	 */
    private String mName;

	/**
	 *
	 */
    private ColumnType mType;

    /**
     * Listeners -----------------------------
     */

    /**
     * Arrays --------------------------------
     */

    /**
     * Booleans ------------------------------
     */

    /**
     * Constructors ==========================
     */

    /**
     * <p>
     * </p>
     *
     * @param name
     */
    public DatabaseEntityColumn(String name) {
        this(name, null);
    }

    /**
     * <p>
     * </p>
     *
     * @param name
     * @param type
     */
    public DatabaseEntityColumn(String name, ColumnType type) {
        this.mName = name;
        this.mType = type;
    }

    /**
     * Methods ===============================
     */

    /**
     * Public --------------------------------
     */

    /**
     * Getters + Setters ---------------------
     */

    /**
     * <p>
     * </p>
     *
     * @return
     */
    public String getName() {
        return mName;
    }

    /**
     * <p>
     * </p>
     *
     * @return
     */
    public ColumnType getType() {
        return mType;
    }

    /**
     * Protected -----------------------------
     */

    /**
     * Private -------------------------------
     */

    /**
     * Abstract methods ----------------------
     */

    /**
     * Inner classes =========================
     */

    /**
     * <h4>Class Overview</h4>
     * <p>
     * </p>
     *
     * @author Martin Albedinsky
     */
    public static class ColumnType {

        /**
         * Constants =============================
         */

        public static final String CONSTRAINT_PRIMARY_KEY = "PRIMARY KEY";

        public static final String CONSTRAINT_AUTOINCREMENT = "AUTOINCREMENT";

        public static final String CONSTRAINT_NOT_NULL = "NOT NULL";

        public static final String CONSTRAINT_UNIQUE = "UNIQUE";

        public static final String CONSTRAINT_CHECK = "CHECK (%s)";

        public static final String CONSTRAINT_DEFAULT = "DEFAULT";

        public static final String CONSTRAINT_COLLATE = "COLLATE";

        private static final String SPACE = " ";

        private static final String KEY_WORD_DEFAULT = " DEFAULT ";

        /**
         * Members ===============================
         */

        String mTypeName;

        private Object mDefaultValue, mTypeConstraint;

        /**
         * Arrays --------------------------------
         */

        private List<String> aConstraints = new ArrayList<String>();

        /**
         * Constructors ==========================
         */

        /**
         * <p>
         * </p>
         *
         * @param typeName
         */
        public ColumnType(String typeName) {
            this(typeName, null);
        }

        /**
         * <p>
         * </p>
         *
         * @param typeName
         * @param defaultValue
         */
        public ColumnType(String typeName, Object defaultValue) {
            this.mTypeName = typeName;
            this.mDefaultValue = defaultValue;
        }

        /**
         * Methods ===============================
         */

        /**
         * Public --------------------------------
         */

        /**
         * <p>
         * </p>
         *
         * @return
         */
        public ColumnType clear() {
            aConstraints.clear();
            mTypeConstraint =  null;
            mDefaultValue = null;
            return this;
        }

        /**
         * <p>
         * </p>
         *
         * @param keyWord
         * @return This type;
         */
        public ColumnType addConstraint(String keyWord) {
            aConstraints.add(keyWord);
            return this;
        }

	    /**
	     * <p>
	     * </p>
	     *
	     * @param keyWords
	     * @return
	     */
	    public ColumnType addConstraints(String... keyWords) {
		    aConstraints.addAll(Arrays.asList(keyWords));
		    return this;
	    }


        /**
         * <p>
         * </p>
         *
         * @param value
         * @return This type.
         */
        public ColumnType addTypeConstraint(Object value) {
            this.mTypeConstraint = value;
            return this;
        }

        /**
         * Getters + Setters ---------------------
         */

        /**
         * <p>
         * </p>
         *
         * @return
         */
        public String getTypeName() {
            return mTypeName;
        }

	    /**
	     * <p>
	     * </p>
	     *
	     * @param value
	     * @return
	     */
	    public ColumnType setDefaultValue(Object value) {
		    this.mDefaultValue = value;
		    return this;
	    }

        /**
         * <p>
         * </p>
         *
         * @return
         */
        public Object getDeafultValue() {
            return mDefaultValue;
        }

        /**
         * <p>
         * </p>
         *
         * @return
         */
        public Object getTypeConstraint() {
            return mTypeConstraint;
        }

        /**
         * <p>
         * </p>
         *
         * @return
         */
        public List<String> getConstraints() {
            return aConstraints;
        }

        /**
         *
         * @return
         */
        public String createType() {
            StringBuilder builder = new StringBuilder(mTypeName);

            // Apply type addConstraint.
            if (mTypeConstraint != null) {
                builder.append("(");
                builder.append(mTypeConstraint.toString());
                builder.append(")");
            }

            // Apply constraints.
            for (String constraint : aConstraints) {
                builder.append(SPACE);
                builder.append(constraint);
            }

            // Check for default value.
            if (mDefaultValue != null) {
                builder.append(KEY_WORD_DEFAULT);
                builder.append(mDefaultValue.toString());
            }
            return builder.toString();
        }

	    /**
	     * Inner classes =========================
	     */

	    /**
	     * <h4>Class Overview</h4>
	     * <p>
	     * </p>
	     *
	     * @author Martin Albedinsky
	     */
	    public static class TEXT extends ColumnType {

		    /**
		     * <p>
		     * </p>
		     */
		    public TEXT() {
			    super("TEXT");
		    }
	    }

	    /**
	     * <h4>Class Overview</h4>
	     * <p>
	     * </p>
	     *
	     * @author Martin Albedinsky
	     */
	    public static class INTEGER extends ColumnType {

		    /**
		     * <p>
		     * </p>
		     */
		    public INTEGER() {
			    super("INTEGER");
		    }
	    }

	    /**
	     * <h4>Class Overview</h4>
	     * <p>
	     * </p>
	     *
	     * @author Martin Albedinsky
	     */
	    public static class REAL extends ColumnType {

		    /**
		     * <p>
		     * </p>
		     */
		    public REAL() {
			    super("REAL");
		    }
	    }

	    /**
	     * <h4>Class Overview</h4>
	     * <p>
	     * </p>
	     *
	     * @author Martin Albedinsky
	     */
	    public static class BLOB extends ColumnType {

		    /**
		     * <p>
		     * </p>
		     */
		    public BLOB() {
			    super("BLOB");
		    }
	    }

	    /**
	     * <h4>Class Overview</h4>
	     * <p>
	     * </p>
	     *
	     * @author Martin Albedinsky
	     */
	    public static class CHECK extends ColumnType {

		    /**
		     * <p>
		     * </p>
		     */
		    public CHECK() {
			    super("CHECK");
		    }
	    }
    }

    /**
     * Interface =============================
     */
}
