/*
 * =================================================================================
 * Copyright (C) 2013 Martin Albedinsky [Wolf-ITechnologies]
 * =================================================================================
 * Licensed under the Apache License, Version 2.0 or later (further "License" only);
 * ---------------------------------------------------------------------------------
 * You may use this file only in compliance with the License. More details and copy
 * of this License you may obtain at
 * 
 * 		http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You can redistribute, modify or publish any part of the code written in this
 * file but as it is described in the License, the software distributed under the 
 * License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES or CONDITIONS OF
 * ANY KIND.
 * 
 * See the License for the specific language governing permissions and limitations
 * under the License.
 * =================================================================================
 */
package com.wit.and.database.entity;

import android.content.ContentValues;

import com.wit.and.database.annotations.entity.EntityName;
import com.wit.and.database.annotations.manage.ContentValue;

import java.lang.reflect.Field;

/**
 * <h4>Class Overview</h4>
 * <p>
 * Description.
 * </p>
 *
 * @author Martin Albedinsky
 */
public class DatabaseEntityModel {

	/**
	 * Constants =============================
	 */

	/**
	 * Log TAG.
	 */
	private static final String TAG = DatabaseEntityModel.class.getSimpleName();

	/**
	 * Indicates if debug private output trough log-cat is enabled.
	 */
	// private static final boolean DEBUG = false;

	/**
	 * Indicates if logging for user output trough log-cat is enabled.
	 */
	// private static final boolean USER_LOG = true;

	/**
	 * Enums =================================
	 */

	/**
	 * Static members ========================
	 */

	/**
	 * Members ===============================
	 */

	/**
	 * Listeners -----------------------------
	 */

	/**
	 * Arrays --------------------------------
	 */

	/**
	 * Booleans ------------------------------
	 */

	/**
	 * Constructors ==========================
	 */

	/**
	 * <p>
	 * </p>
	 */
	public DatabaseEntityModel() {
	}

	/**
	 * Methods ===============================
	 */

	/**
	 * Public --------------------------------
	 */

	/**
	 * <p>
	 * </p>
	 *
	 * @return
	 */
	public ContentValues createContentValues() {
		return toContentValues(this);
	}

	/**
	 * <p>
	 * </p>
	 *
	 * @return
	 */
	public ContentValues toContentValues(Object model) {
		return this.buildContentValuesFrom(model);
	}

	/**
	 * Getters + Setters ---------------------
	 */

	/**
	 * Protected -----------------------------
	 */

	/**
	 * Private -------------------------------
	 */

	/**
	 *
	 * @param model
	 * @return
	 */
	private ContentValues buildContentValuesFrom(Object model) {
		final Class<?> classOfModel = model.getClass();
		final ContentValues contentValues = new ContentValues();

		// Obtain name of entity to which is this model assigned.
		String entityName = null;
		if (classOfModel.isAnnotationPresent(EntityName.class)) {
			entityName = classOfModel.getAnnotation(EntityName.class).value();
		}

		/**
		 * Process all annotated members of the given model.
		 */
		Field[] fields = classOfModel.getDeclaredFields();
		for (Field field : fields) {
			if (field.isAnnotationPresent(ContentValue.class)) {
				// Set field accessible to successfully obtain its value.
				field.setAccessible(true);

				ContentValue contentValue = field.getAnnotation(ContentValue.class);
				String key = (entityName != null) ? entityName + "_" : "";

				// Key for content value can be presented in the annotation, if not
				// the name of field will be key.
				if (contentValue.value().length() > 0) {
					key += contentValue.value();
				} else {
					key += field.getName();
				}

				try {
					final Object fieldValue = field.get(model);

					// Check for enum field.
					if (fieldValue instanceof Enum) {
						// Add into content values as enum ordinal representation.
						contentValues.put(key, ((Enum) fieldValue).ordinal());
					} else {
						// Add into content values as string representation.
						contentValues.put(key, fieldValue.toString());
					}
				} catch (IllegalAccessException e) {
					e.printStackTrace();
				}
			}
		}
		return contentValues;
	}

	/**
	 * Abstract methods ----------------------
	 */

	/**
	 * Inner classes =========================
	 */

	/**
	 * Interface =============================
	 */
}
