/*
 * =================================================================================
 * Copyright (C) 2013 Martin Albedinsky [Wolf-ITechnologies]
 * =================================================================================
 * Licensed under the Apache License, Version 2.0 or later (further "License" only);
 * ---------------------------------------------------------------------------------
 * You may use this file only in compliance with the License. More details and copy
 * of this License you may obtain at
 * 
 * 		http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You can redistribute, modify or publish any part of the code written in this
 * file but as it is described in the License, the software distributed under the 
 * License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES or CONDITIONS OF
 * ANY KIND.
 * 
 * See the License for the specific language governing permissions and limitations
 * under the License.
 * =================================================================================
 */
package com.wit.and.database.manage;

import android.util.Log;

import com.wit.and.database.annotations.entity.EntityName;
import com.wit.and.database.annotations.manage.Columns;
import com.wit.and.database.exception.MissingAnnotationException;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * <h4>Class Overview</h4>
 * <p>
 * </p>
 *
 * @author Martin Albedinsky
 */
public class EntityProjection {

	/**
	 * Constants =============================
	 */

	/**
	 * Log TAG.
	 */
	private static final String TAG = EntityProjection.class.getSimpleName();

	/**
	 * Indicates if debug private output trough log-cat is enabled.
	 */
	// private static final boolean DEBUG = false;

	/**
	 * Indicates if logging for user output trough log-cat is enabled.
	 */
	// private static final boolean USER_LOG = true;

	/**
	 * Enums =================================
	 */

	/**
	 * Static members ========================
	 */

	/**
	 * Members ===============================
	 */

	/**
	 *
	 */
	private String mTableName;

	/**
	 * Listeners -----------------------------
	 */

	/**
	 * Arrays --------------------------------
	 */

	/**
	 *
	 */
	private List<String> aColumns = new ArrayList<String>();

	/**
	 * Booleans ------------------------------
	 */

	/**
	 * Constructors ==========================
	 */

	/**
	 * <p>
	 * </p>
	 *
	 * @throws com.wit.and.database.exception.MissingAnnotationException
	 */
	public EntityProjection() {
		final Class<?> projectionClass = getClass();
		/**
		 * Process class annotations.
		 */

		Log.d(TAG, "checking projection class("+projectionClass+")");

		// Retrieve table name.
		if (projectionClass.isAnnotationPresent(EntityName.class)) {
			this.mTableName = projectionClass.getAnnotation(EntityName.class).value();
		}
		// Check if we can retrieve table name from the enclosing class.
		// This can be helpful when the projection class is as member
		// class of its table class.
		else if (projectionClass.isMemberClass()) {
			Class<?> enclosingClass = projectionClass.getEnclosingClass();
			if (enclosingClass != null && enclosingClass.isAnnotationPresent(EntityName.class)) {
				this.mTableName = enclosingClass.getAnnotation(EntityName.class).value();
			} else {
				throw new MissingAnnotationException("Can't create empty table projection without table name.");
			}
		}

		// Retrieve columns.
		if (projectionClass.isAnnotationPresent(Columns.class)) {
			columns(projectionClass.getAnnotation(Columns.class).value());
		}
	}

	/**
	 * <p>
	 * </p>
	 *
	 * @param tableName
	 */
	public EntityProjection(String tableName) {
		this.mTableName = tableName;
	}

	/**
	 * Methods ===============================
	 */

	/**
	 * Public --------------------------------
	 */

	/**
	 * <p>
	 * </p>
	 *
	 * @param tableName
	 * @param columnName
	 * @return
	 */
	public static String createProjectionRow(String tableName, String columnName) {
		return tableName + "." + columnName;
	}

	/**
	 * <p>
	 * </p>
	 *
	 * @param projectionColumn
	 * @return
	 */
	public EntityProjection column(String projectionColumn) {
		aColumns.add(projectionColumn);
		return this;
	}

	/**
	 * <p>
	 * </p>
	 *
	 * @param projectionColumns
	 * @return
	 */
	public EntityProjection columns(String... projectionColumns) {
		aColumns.addAll(Arrays.asList(projectionColumns));
		return this;
	}

	/**
	 * <p>
	 * </p>
	 *
	 * @return
	 */
	public String[] create() {
		String[] projection = new String[aColumns.size()];
		for (int i = 0; i < aColumns.size(); i++) {
			projection[i] = createProjectionRow(mTableName, aColumns.get(i));
		}
		return projection;
	}

	/**
	 * Getters + Setters ---------------------
	 */

	/**
	 * <p>
	 * </p>
	 *
	 * @return
	 */
	public String getTableName() {
		return mTableName;
	}

	/**
	 * <p>
	 * </p>
	 *
	 * @return
	 */
	public List<String> getColumns() {
		return aColumns;
	}

	/**
	 * Protected -----------------------------
	 */

	/**
	 * Private -------------------------------
	 */

	/**
	 * Abstract methods ----------------------
	 */

	/**
	 * Inner classes =========================
	 */

	/**
	 * Interface =============================
	 */
}
