/*
 * =================================================================================
 * Copyright (C) 2013 Martin Albedinsky [Wolf-ITechnologies]
 * =================================================================================
 * Licensed under the Apache License, Version 2.0 or later (further "License" only);
 * ---------------------------------------------------------------------------------
 * You may use this file only in compliance with the License. More details and copy
 * of this License you may obtain at
 * 
 * 		http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You can redistribute, modify or publish any part of the code written in this
 * file but as it is described in the License, the software distributed under the 
 * License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES or CONDITIONS OF
 * ANY KIND.
 * 
 * See the License for the specific language governing permissions and limitations
 * under the License.
 * =================================================================================
 */
package com.wit.and.database.sql;

import android.util.Log;

import com.wit.and.database.entity.DatabaseEntityColumn;

import java.util.ArrayList;
import java.util.List;

/**
 * <h4>Class Overview</h4>
 * <p>
 * Description.
 * </p>
 *
 * @author Martin Albedinsky
 */
public class SQLCreateQuery extends SQLQuery {

    /**
     * Constants =============================
     */

    /**
     * Log TAG.
     */
    private static final String TAG = SQLCreateQuery.class.getSimpleName();

    /**
     * Indicates if debug private output trough log-cat is enabled.
     */
    private static final boolean DEBUG = true;

    /**
     * Indicates if logging for user output trough log-cat is enabled.
     */
    // private static final boolean USER_LOG = true;

    public static final String KEY_WORD_CREATE_TABLE = "CREATE TABLE IF NOT EXISTS";

    /**
     * Enums =================================
     */

    /**
     * Static members ========================
     */

    /**
     * Members ===============================
     */

    /**
     * Listeners -----------------------------
     */

    /**
     * Arrays --------------------------------
     */

    /**
     *
     */
    private List<DatabaseEntityColumn> aColumns = new ArrayList<DatabaseEntityColumn>();

    /**
     * Booleans ------------------------------
     */

    /**
     * Constructors ==========================
     */

    /**
     * <p>
     * Same as {@link com.wit.and.database.sql.SQLQuery#SQLQuery(String)}.
     * </p>
     *
     * @param entityName
     */
    public SQLCreateQuery(String entityName) {
        super(entityName);
    }

    /**
     * Methods ===============================
     */

    /**
     * Public --------------------------------
     */

    /**
     *
     * @param column
     * @return This query.
     */
    public SQLCreateQuery column(DatabaseEntityColumn column) {
        aColumns.add(column);
        return this;
    }

    /**
     *
     * @param columns
     * @return This query.
     */
    public SQLCreateQuery columns(List<DatabaseEntityColumn> columns) {
        aColumns.clear();
        aColumns.addAll(columns);
        return this;
    }

	/**
	 * <p>
	 * </p>
	 *
	 * @return
	 */
	public boolean hasColumns() {
		return aColumns.size() != 0;
	}

    /**
     */
    @Override
    public String buildQuery() {
        QUERY_BUILDER.setLength(0);

        // Create table by its name.
        QUERY_BUILDER.append(KEY_WORD_CREATE_TABLE).append(" ").append(getEntityName()).append(" (");

        // Create all entity columns.
        final int n = aColumns.size();
        for (int i = 0; i < n; i++) {
            DatabaseEntityColumn column = aColumns.get(i);

            // Create the column.
            QUERY_BUILDER.append(column.getName());
            QUERY_BUILDER.append(" ");
            QUERY_BUILDER.append(column.getType().createType());

            if (i != (n - 1)) {
                // Add the separator.
                QUERY_BUILDER.append(", ");
            }
        }
        // End the query.
        QUERY_BUILDER.append(")");

        if (DEBUG)
            Log.d(TAG, "Created query('" + QUERY_BUILDER.toString() + "')");

        // Build query.
        return QUERY_BUILDER.toString();
    }

    /**
     * Getters + Setters ---------------------
     */

    /**
     *
     * @return
     */
    public List<DatabaseEntityColumn> getColumns() {
        return aColumns;
    }

    /**
     * Protected -----------------------------
     */

    /**
     * Private -------------------------------
     */

    /**
     * Abstract methods ----------------------
     */

    /**
     * Inner classes =========================
     */

    /**
     * Interface =============================
     */
}
