/*
 * =================================================================================
 * Copyright (C) 2013 Martin Albedinsky [Wolf-ITechnologies]
 * =================================================================================
 * Licensed under the Apache License, Version 2.0 or later (further "License" only);
 * ---------------------------------------------------------------------------------
 * You may use this file only in compliance with the License. More details and copy
 * of this License you may obtain at
 * 
 * 		http://www.apache.org/licenses/LICENSE-2.0
 * 
 * You can redistribute, modify or publish any part of the code written in this
 * file but as it is described in the License, the software distributed under the 
 * License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES or CONDITIONS OF
 * ANY KIND.
 * 
 * See the License for the specific language governing permissions and limitations
 * under the License.
 * =================================================================================
 */
package com.wit.and.examples.database.entity;

import android.content.Context;

import com.wit.and.database.annotations.DatabaseAuthority;
import com.wit.and.database.annotations.entity.EntityColumn;
import com.wit.and.database.annotations.entity.EntityLoader;
import com.wit.and.database.annotations.entity.EntityName;
import com.wit.and.database.annotations.entity.EntityTransactions;
import com.wit.and.database.annotations.manage.Columns;
import com.wit.and.database.annotations.manage.Sort;
import com.wit.and.database.entity.DatabaseEntityColumn;
import com.wit.and.database.manage.EntityProjection;
import com.wit.and.database.manage.LoaderOptions;
import com.wit.and.database.entity.DatabaseEntity;
import com.wit.and.examples.database.DatabaseProvider;
import com.wit.and.examples.database.model.Customer;

/**
 * @author Martin Albedinsky
 */
@EntityName(EntityCustomers.ENTITY_NAME)
@EntityTransactions(EntityCustomersTransactions.class)
@DatabaseAuthority(DatabaseProvider.DATABASE_AUTHORITY)
public class EntityCustomers extends DatabaseEntity<EntityCustomersTransactions> implements EntityCustomersTransactions {

	// Name for database entity.
	public static final String ENTITY_NAME = "customers";

	@EntityColumn(
			value = DatabaseEntityColumn.ColumnType.INTEGER.class,
			constraints = {DatabaseEntityColumn.ColumnType.CONSTRAINT_PRIMARY_KEY, DatabaseEntityColumn.ColumnType.CONSTRAINT_AUTOINCREMENT})
	public static final String COLUMN_ID = "id";

	@EntityColumn
	public static final String COLUMN_FIRST_NAME = "first_name";

	@EntityColumn
	public static final String COLUMN_LAST_NAME = "last_name";

	/**
	 * Table loader ids.
	 */
	public static final int LOADER_CUSTOMERS = 0x01;


	/**
	 * Table loader options.
	 */
	@EntityLoader(LOADER_CUSTOMERS)
	@Sort(COLUMN_FIRST_NAME + " ASC")
	private final LoaderOptions LOADER_CUSTOMERS_OPTIONS = new LoaderOptions().projection(new ProjectionCustomers());

	/**
	 * Table projections.
	 */

	@Columns({COLUMN_ID, COLUMN_FIRST_NAME, COLUMN_LAST_NAME})
	private final class ProjectionCustomers extends EntityProjection {}

	public EntityCustomers() {
		super();
	}

	public EntityCustomers(Context context) {
		super(context);
	}

	@Override
	public void createCustomer(Customer customer) {
		insert(customer);
	}
}

